#include "common.h"
#ifdef FUNCTION_PROFILE
#include "functable.h"
#endif

#ifdef XDOUBLE
#define  FABS	fabsl
#define  GAM    0x1p12L
#define  GAMSQ  0x1p24L
#define  RGAMSQ 0x1p-24L
#elif defined(DOUBLE)
#define  FABS	fabs
#define  GAM    0x1p12
#define  GAMSQ  0x1p24
#define  RGAMSQ 0x1p-24
#else
#define  FABS	fabsf
#define  GAM    0x1p12F
#define  GAMSQ  0x1p24F
#define  RGAMSQ 0x1p-24F
#endif

#ifndef CBLAS

void NAME(FLOAT *dd1, FLOAT *dd2, FLOAT *dx1, FLOAT *DY1, FLOAT *dparam){

  FLOAT	dy1 = *DY1;

#else

void CNAME(FLOAT *dd1, FLOAT *dd2, FLOAT *dx1, FLOAT dy1, FLOAT *dparam){

#endif

  FLOAT du, dp1, dp2, dq1, dq2, dh11, dh12, dh21, dh22, dtemp;
  int   flag;

#ifndef CBLAS
  PRINT_DEBUG_NAME;
#else
  PRINT_DEBUG_CNAME;
#endif

  flag = 0;
  dh11 = ZERO;
  dh12 = ZERO;
  dh21 = ZERO;
  dh22 = ZERO;


  if (*dd1 < ZERO) {
    flag = -1;
    *dd1 = ZERO;
    *dd2 = ZERO;
    *dx1 = ZERO;
  } else {
    dp2 = *dd2 * dy1;
    if (dp2 == ZERO) {
      flag = -2;
      goto L260;
    }
    dp1 = *dd1 * *dx1;
    dq2 = dp2 * dy1;
    dq1 = dp1 * *dx1;
    
    if (FABS(dq1) > FABS(dq2)) {
      dh21 = -(dy1) / *dx1;
      dh12 = dp2 / dp1;
      
      du = ONE - dh12 * dh21;
      
      if (du > ZERO) {
	flag = 0;
	*dd1 /= du;
	*dd2 /= du;
	*dx1 *= du;
      }
    } else {
      if (dq2 < ZERO) {
	flag = -1;
	*dd1 = ZERO;
	*dd2 = ZERO;
	*dx1 = ZERO;
      } else {
	flag = 1;
	dh11 = dp1 / dp2;
	dh22 = *dx1 / dy1;
	du = ONE + dh11 * dh22;
	dtemp = *dd2 / du;
	*dd2 = *dd1 / du;
	*dd1 = dtemp;
	*dx1 = dy1 * du;
      }
    }
    
    if (*dd1 != ZERO) {
      while(*dd1 <= RGAMSQ || *dd1 >= GAMSQ) {
	if (flag == 0) {
	  dh11 = ONE;
	  dh22 = ONE;
	  flag = -1;
	} else {
	  dh21 = -ONE;
	  dh12 = ONE;
	  flag = -1;
	}
	if (*dd1 <= RGAMSQ) {
	  *dd1 *= GAMSQ;
	  *dx1 /= GAM;
	  dh11 /= GAM;
	  dh12 /= GAM;
	} else {
	  *dd1 /= GAMSQ;
	  *dx1 *= GAM;
	  dh11 *= GAM;
	  dh12 *= GAM;
	}
      }
    }
    if (*dd2 != ZERO) {
      while(FABS(*dd2) <= RGAMSQ || FABS(*dd2) >= GAMSQ) {
	if (flag == 0) {
	  dh11 = ONE;
	  dh22 = ONE;
	  flag = -1;
	} else {
	  dh21 = -ONE;
	  dh12 = ONE;
	  flag = -1;
	}
	if (FABS(*dd2) <= RGAMSQ) {
	  *dd2 *= GAMSQ;
	  dh21 /= GAM;
	  dh22 /= GAM;
	} else {
	  *dd2 /= GAMSQ;
	  dh21 *= GAM;
	  dh22 *= GAM;
	}
      }
    }
  }
  if (flag < 0) {
    dparam[1] = dh11;
    dparam[2] = dh21;
    dparam[3] = dh12;
    dparam[4] = dh22;
  } else if (flag == 0) {
    dparam[2] = dh21;
    dparam[3] = dh12;
  } else {
    dparam[1] = dh11;
    dparam[4] = dh22;
  }
L260:
  dparam[0] = (FLOAT)flag;
  return;
}

